%% Matlab seminar 2016
% Example 2: Discrete Frequency domain
% ********************************************************************
% Javier Gomez Bolanos 2016 - Dept. Signal Processing and Acoustics
%                   Aalto University
% ********************************************************************
% This is an example of Matlab script that generates two sinusoidals and
% process it to obtain different parameters from the signal.
% Each process is divided in Sections which are explained separately
% Section 1: Generate data
% Section 2: FFT
% Section 3: Calculate RMS in frequency domain
clear all; clc;

% ***********************************************************************
%% Section 1: Generate data
% generating two sine signal with frequency [f0 f1] and duration s_length
% sampled at fs frequency rate (generate it as column vectors):
fs = 48e3; % sampling rate (Hz);
f = [1e2 1e3]; % frequency of the signal 1 and 2 (Hz)
s_length = 1.5; % duration in seconds
inc_time = 1/fs; % time intervals between samples
N = ceil(s_length.*fs); % length of the vector in samples
time_axis = (0:N-1).'.*inc_time; % time axis vector (column vector)
s = zeros(N,size(f,2)); % Initialize variable
% in a FOR loop
for i=1:size(f,2)    
    s(:,i) = sin(2*pi.*time_axis.*f(i)); % generate data matrix
end;
% in a matrix calculation
time_axis2 = repmat(time_axis,1,size(f,2)); % repeat the time_axis vector 
f_2 = repmat(f,size(time_axis,1),1);% repeat the f vector
s = sin(2*pi.*time_axis2.*f_2);

% ***********************************************************************
%% FFT
Nfft = 2^12; % size of the FFT
inc_freq = fs/Nfft; % frequency bin size
S0 = fft(s,Nfft); % FFT 
S1 = fft(s,Nfft)/Nfft; % FFT and normalization
N1=Nfft/2; % short length of signal
S2 = fft(s(1:N1,:),Nfft)/N1; % zero-pading FFT and normalization
win = hanning(N1);
win = repmat(win./rms(win),1,size(s,2));
S3 = fft(s(1:N1,:).*win,Nfft)/N1; % zero-pading FFT and normalization

% ***********************************************************************
%% Plotting
freq_axis = (0:Nfft-1).*inc_freq; % Frequency vector axis
figure(1);clf;
semilogx(freq_axis,20*log10(abs([sum(S0,2) sum(S1,2)...
    sum(S2,2) sum(S3,2)]))); % Plot in semilogarithmic scale
grid on; % Enable grid
xlim([20 48000-20]);ylim([-50 70]);
xlabel('Frequency (Hz)'); ylabel('Amplitude non corrected');
legend('Non FFT normalization','Normalized N=Nfft',...
    'Normalized zero-padding','Normalized windowing');
% plotting the peak amplitude
figure(2);clf;
semilogx(freq_axis,20*log10(abs([sum(S1,2) sum(S2,2) sum(S3,2)]).*2)); 
grid on; % Enable grid
xlim([20 20e3]);ylim([-30 0]);
xlabel('Frequency (Hz)'); ylabel('Amplitude');
legend('Normalized N=Nfft','Normalized zero-padding',...
    'Normalized windowing');
% plotting the rms value
figure(3);clf;
semilogx(freq_axis,20*log10(abs([sum(S1,2) sum(S2,2)...
    sum(S3,2)]).*2./sqrt(2)));
grid on; % Enable grid
xlim([20 20e3]);ylim([-30 0]);
xlabel('Frequency (Hz)'); ylabel('Normalized Magnitude RMS');
legend('Normalized N=Nfft','Normalized zero-padding',...
    'Normalized windowing');