%% Matlab seminar 2016
% Example 3: Calibration and Absolute pressure from impulse responses
% ********************************************************************
% Javier Gomez Bolanos 2016 - Dept. Signal Processing and Acoustics
%                   Aalto University
% ******************************************************************** This
% is an example of Matlab script that generates a scaled sinusoidal to be
% used as calibration signal and loads the sweep response of a loudspeaker
% to obtain different parameters from the loudspeaker. Each process is
% divided in Sections which are explained separately Section 1:
% Generate/Load data Section 2: Calibration Section 3: Calculate parameters
clear all; clc;

%% Generate/Load data
% sine signal
fs = 48e3; % sampling rate (Hz)
f0 = 1e3; % frequency of the signal (Hz)
s_length = 10; % duration in seconds
s1 = 0.0043.*sine(f0,s_length,fs).'; % Assume this signal to be the signal
% of a calibrator (94dBSPL/1kHz) is measured by the microphone

% Loudspeaker response
load('LS_response_sweep.mat'); % x contains two vectors: the measured
% sweep and the inverse sweep spectra calculated for a sweep of 1Vrms
ls_sweep = x(:,1);
inv_sweep = x(:,2);

% ***********************************************************************
%% Calibration to a sensitivity of 1 Vrms

% Calculate the rms of the signal
g_cal = rms(s1); % g_cal represents the sensitivity of the measurement
% chain (mic+preamp+DAQ)
% ******* Q1: why is g_cal so important?

% Calibrate the input signal
ls_sweep = ls_sweep./g_cal; % after this, if the input signal is a
% 1kHz sine signal with 94dBSPL the rms value of the signal is 1Vrms

% ***********************************************************************
%% Calculate parameters

% Impulse response (exp-sweep method)
LS_sweep = fft(ls_sweep,size(inv_sweep,1));
FR = LS_sweep.*inv_sweep; % FFT to the same size of inv_sweep
ir = fftshift(ifft(FR)); % inverse Fourier transform
% ******** Q2: which 'symmetric' FFT?
ir_lin = ir(end/2+1:end); % linear part of IR
ir_har = ir(1:end/2); % Harmonics of IR (there is a way to separate each
% harmonic. You can read Farina's paper on sweeps to know how to implement
% the sweep and separate each component of the reponse

% Frequency response of loudspeaker
Nfft = 2^13; % size of the FFT 
% ******** Q3: which size is the best?
inc_freq = fs/Nfft; % frequency bin size (increment of frequency)
FR_ls = fft(ir_lin,Nfft);

% Sensitivity calculated with broadband spectra
freq_axis = inc_freq.*(0:Nfft-1);
S1 = 20*log10(abs(sqrt(sum(abs(FR_ls).^2)/Nfft)));
% using the impulse response
S2 = sqrt(sum(abs(ir_lin).^2)); % S1 and S2 should be equal 
% ******** Q4: Why S1 and S2 should be equal?


% Excess group delay
[~,ir_min] = rceps(ir_lin);
FR_min = fft(ir_min,Nfft);
FR_max = FR_ls./FR_min;
% excess group delay in seconds
GD_excess =[angle(FR_max(1)); -diff(unwrap(angle(FR_max)))]./(2*pi*inc_freq);
% ******** Q5: Why "angle(FR_max(1))" is added at the beginning of
% GD_excess?
%% Plotting

figure(1);clf;
 % Plot in semilogarithmic scale
semilogx(freq_axis,20*log10(abs(FR_ls)/2e-5),'Linewidth',2);
grid on; % Enable grid
xlim([20 20e3]);ylim([75 95]);
xlabel('Frequency (Hz)'); ylabel('Sound Pressure Level (dB)');
% ******** Q6: Why plotting with logarithmic frequency axes?
% ******** Q7: Why plotting with logarithmic SPL axes?
figure(2);clf;
time_axis = (0:size(ir_lin,1)-1)./fs; % time axis
plot(time_axis(1:2048).*1e3,ir_lin(1:2048),'Linewidth',2);
grid on; % Enable grid
xlabel('Time (ms)'); ylabel('Gain');

figure(3);clf;
semilogx(freq_axis,GD_excess.*1e3,'Linewidth',2); % GD in milliseconds
grid on; % Enable grid
xlim([20 20e3]);ylim([0 4]);
xlabel('Frequency (Hz)'); ylabel('Excess delay (ms)');

