%% Matlab seminar 2016
% Example 4: Schroeder integration
% ********************************************************************
% Javier Gomez Bolanos 2016 - Dept. Signal Processing and Acoustics
%                   Aalto University
% ********************************************************************
% This is a Matlab script that load the impulse response of a room and
% calculates the Schroeder integral to determine the reverberation time
clear all; clc;
% ********************************************************************
%% Load data
fs = 48000; % Sampling Rate
load('RoomImpulseResponse.mat'); % load data
% ******** Q1: Why not to assign a variable, eg. x = load('xxxx.mat');?
time_axis = (0:size(x,1)-1).'./fs; % time axis in seconds
% ********************************************************************
%% Calculate Schroeder curve
xx=x.^2; % square the response
% with FOR loop
sch1=xx; % initialize output vector
for i=size(x,1)-1:-1:1 % reverse integration
    sch1(i)=sch1(i+1)+xx(i);
end;
% with CUMSUM
sch2 = cumsum(xx,1,'reverse');

% ********************************************************************
%% Plotting

figure(1);clf;
plot(time_axis,20*log10(abs(x)),'b',time_axis,10*log10(sch1),'r',...
    time_axis,10*log10(sch2./max(sch2).*max(abs(x).^2)),'m','Linewidth',2);
xlabel('Time (s)'); ylabel('Level');
grid on;
legend('RIR','Schroeder 1', 'Schroeder 2 (Scaled)');
% You can use polyfit to make a linear regression of the Schroeder curve in
% order to obtain the slope of the curve for automatic calculation of RT.

% **** Q2: Why sch1 and sch2 are plotted with 10*log10 instead of 20*log10?


