function H = mag_smoothing(B)
%% Matlab seminar 2016
% Example 5: Smoothing algorithm
% ********************************************************************
% Javier Gomez Bolanos 2016 - Dept. Signal Processing and Acoustics
%                   Aalto University
% ********************************************************************
% This is a Matlab function that implements the 1/3 octave smoothing of the
% input magnitude response B.

stop=length(B)/2+1; % We don't need the other side of spectra
smooth_factor=1/3; % 1/3 octave bandwidth
Q=sqrt(2^(smooth_factor))/(2^(smooth_factor)-1); % Q-factor
B1=abs(B); % Only magnitude
H=B1(1:stop); % Initialize output vector
q=fix(Q)+1; % pointer to first frequency to be smoothed
for i=q:stop % scan each frequency bin
    
    N=fix(i/Q); % ammount of frequency bins in the smoothing window 
    if mod(N,2)==0 % detect if it is even
        N=N+1;
    end;
    fc=(N+1)/2; % center frequency
    fh=N-fc;
    
    if i+fh>stop % detect if there is not enough bandwidth
        f=N-(i+fh-stop);
    else
        f=N;
    end;
    
    H(i)=sqrt(sum(B1(i-fh:i+fh-(N-f)).^2)/f); % smoothing
end;
H = [H; H(end-1:-1:2)];
% ******** Q1: What kind of frequency window can be used?
% ******** Q2: Why negative spectra is not used in the calculations?
% ******** Q3: Why phase is not used in the smoothing?

